==> musashi-core-1.0.3.tar.gz: musashi-core-1.0.3/exp/xtbsupdate.c <==
/*============================================================================*/
/* ѹ                                                                   */
/*----------------------------------------------------------------------------*/
/* 0.1 : ǿ 2003/06/20                                              */
/* 0.2 : ץͿѹ 2003/08/18                                  */
/* 0.3 : եå(¾å)ǽɲ 2003/09/26                        */
/*============================================================================*/

#include <musashi.h>
#include <stdlib.h>
#include <string.h>
#include <sys/file.h>

#include <xtbsupdateHelp.h>
struct mssComHelp comHelp={
  "xtbsupdate",   /* ޥ̾       */
  "0.3",          /* С       */
  HELPT,          /* ޥɥȥ */
  HELPS,          /*              */
  HELPE,          /*            */
  HELPR,          /* ȥޥ     */
  HELPA,          /* Ծ         */
  HELPB,          /* ХݡȾ */
  HELPH           /* ۡڡ     */
};

extern struct mssGlobalVariables mssGV;

/**
 * ϰϥι¤
 * ϰϤǤΤϡǸʸΤ
 * ex) 
 */
struct RangeKey {
  int   *fldNo; /*ƹܤֹ*/
  char  **str;
  char  *from;
  char  *to;
  int    cnt;
};

int keyCmp(char **str1, char **str2, struct RangeKey *rkey){
  int i;
  int cmp=0;

  for(i=0; i<rkey->cnt; i++){
    cmp=strcmp( *(str1+*(rkey->fldNo+i)), *(str2+*(rkey->fldNo+i)) );
               /* ǡ         -kιܥץ */
    if(cmp!=0) return(cmp);
  }
  return(cmp);
}

int rngKeyCmp(char **str, struct RangeKey *rkey){
  int i;
  int cmp=0;

  for(i=0; i<rkey->cnt; i++){
    if( MssIsNull(*(str+*(rkey->fldNo+i))) ) return(1);
    cmp=strcmp( *(rkey->str+i), *(str+*(rkey->fldNo+i)) );
               /* ǡ         -kιܥץ */
    if(cmp!=0) return(cmp);
  }
  return(cmp);
}

/**
 * Ϳ줿쥳(pnt)ϰϥ(rkey)ˤ뤫ɤå
 * ϰʤ1,ǤʤУ֤
 * ǡNULLʤ飰֤
 */
int inRange(char **pnt,struct RangeKey *rkey){
  int i;
  int cmp;
  int cmpFrom;
  int cmpTo;

  /*ϰϹܰʳunmatchʤ0֤*/
  for(i=0; i<rkey->cnt-1; i++){
    if( MssIsNull(*(pnt+*(rkey->fldNo+i))) ) return(0);
    cmp=strcmp( *(rkey->str+i), *(pnt+*(rkey->fldNo+i)) );
    if(cmp!=0) return(0);
  }

  /*ϰϹʤ1֤*/
  if( MssIsNull(*(pnt+*(rkey->fldNo+i))) ) return(0);
  cmpFrom=strcmp( rkey->from, *(pnt+*(rkey->fldNo+i)) );
  cmpTo  =strcmp( rkey->to,   *(pnt+*(rkey->fldNo+i)) );
  if( cmpFrom<=0 && cmpTo>=0){
    return(1);
  }else{
    return(0);
  }
}

/**
 * ǡ顢ϰϥƬ֤֤
 */
int bSearch(struct RangeKey *rkey,struct mssFPR *fp,int fldCnt,int recCnt,int datTop,int recLen){

  struct mssFldRec    *fr1;  /*-ԥХåե¤             */
  struct mssFldRec    *fr2;  /*-ԥХåե¤             */
  int l=0;
  int r=recCnt-1;
  int x;
  int cmp;

  fr1 =mssInitFldRec(fldCnt);
  fr2 =mssInitFldRec(fldCnt);

  while(r>=l){
    x=(l+r)/2;
    mssReadRandFldRec(fp,fr1,x,datTop,recLen);
    mssGV.inCnt++;
    cmp=rngKeyCmp(fr1->pnt,rkey);

         /* v<x */
         if(cmp<0) r=x-1;
         /* v>x */
    else if(cmp>0) l=x+1;
         /* v=x */
    else          {
      if(x==0) return(x);
      mssReadRandFldRec(fp,fr2,x-1,datTop,recLen);
      if(0!=keyCmp(fr1->pnt, fr2->pnt, rkey)){
        mssFreeFldRec(fr1);
        mssFreeFldRec(fr2);
        return(x);
      }else{
        r=x-1;
      }
    }
  }
  mssFreeFldRec(fr1);
  mssFreeFldRec(fr2);
  return(l);
}

int main(int argc, char *argv[]){
/*============================================================================*/
/* ץ                                                       */
/*============================================================================*/
/*----------------------------------------------------------------------------*/
/*                                                                    */
/*----------------------------------------------------------------------------*/
  MssOptKEY optKEY={
    OKEY,   /* ץ󥿥                                             */
    "k",    /* (ʣʸԲ)                                   */
    0,      /* 0:ץ, 1:ɬ, 2:XMLtableǤΤɬ(txtǤ̵)      */
    MssFieldMaxCnt, /* ǽʺܿ                                 */
    "i",    /* оݤȤϥǡΥ(GUI)                  */
    1,      /* ǥե(Υץ󤬻ꤵʤäȤư) */
            /* 1:ƤιԤۤ륭ͤȤư                             */
            /* 2:ƤιԤƱͤȤư)                            */
    KEYT,   /* ΥץΥȥ(Helpɽ)                         */
    KEYC    /* ΥץΥ(Helpɽ)                         */
  };

/*----------------------------------------------------------------------------*/
/* ʸꥹ                                                           */
/*----------------------------------------------------------------------------*/
  MssOptSRL optVAL={
    OSRL,   /* ץ󥿥                                             */
    "v",    /* (ʣʸԲ)                                   */
    1,      /* 0:ץ, 1:ɬ, 2:XMLtableǤΤɬ(txtǤ̵)      */
    NULL,   /* ǥե                                                   */
    100,    /*()κ                                      */
    MssFieldMaxCnt,/*Ǥ(޶)κ                       */
    1,      /*ͤκǾʸ                                                */
    MssFieldMaxLen,/*ͤκʸ                                         */
    1,      /*ϰϻǽե饰                                            */
    VALT,   /* ΥץΥȥ(Helpɽ)                         */
    VALC    /* ΥץΥ(Helpɽ)                         */
  };

/*----------------------------------------------------------------------------*/
/*                                                                    */
/*----------------------------------------------------------------------------*/
  MssOptFLD optFLD={
    OFLD,   /* ץ󥿥                                             */
    "f",    /* (ʣʸԲ)                                   */
    1,      /* 0:ץ, 1:ɬ, 2:XMLtableǤΤɬ(txtǤ̵)      */
    MssFieldMaxCnt, /* ǽʺܿ                                 */
    "i",    /* оݤȤϥǡΥ(GUI)                  */
    0,      /* ɽĤ뤫ɤ(0:Բ,1:)                      */
    0,      /* ̾Ǥ뤫ɤ(0:Բ,1:)                    */
    NULL,   /* ܥץ(%ʲ)ǻǽʸ                        */
            /* ex) ԲĤξNULL, "nr": "-f ̾%rn"λǽ     */
    FLDT,   /* ΥץΥȥ(Helpɽ)                         */
    FLDC,   /* ΥץΥ(Helpɽ)                         */
    NULL    /* ե饰ˤĤƤ(Helpɽ)ʣξϥޤǶڤ   */
  };

/*----------------------------------------------------------------------------*/
/* ϥե                                                               */
/*----------------------------------------------------------------------------*/
  MssOptINF optINF={
    OINF,   /* ץ󥿥                                             */
    "i",    /* (ʣʸԲ)                                   */
    1,      /* 0:ץ, 1:ɬ                                         */
    1,      /* ǽκե                                     */
    0,      /*1:file not foundΥ顼ǽλʤ 0:                   */
    INFT,   /* ΥץΥȥ(Helpɽ)                         */
    INFC    /* ΥץΥ(Helpɽ)                         */
  };

/*----------------------------------------------------------------------------*/
/* ץޤȤ                                                       */
/*----------------------------------------------------------------------------*/
  void *opt[]={&optKEY,&optVAL,&optFLD,&optINF,NULL};

/*============================================================================*/
/* ѿ                                                             */
/*============================================================================*/
  struct mssHeader    *hdi; /*ϥե<head>Ǽ¤*/
  struct mssFPR       *fpr; /*ϥե빽¤                */
  struct mssFldRec    *fr;  /*-ԥХåե¤             */

  int recLen;     /*쥳Ĺ*/
  int recCnt;     /*쥳ɿ*/
  int datTop;     /*ǡΤƬΰ*/
  int recNum;     /*̤ȤƤΥ쥳ֹ*/
  /*int flstat;*/     /*եåΥơ*/

  struct RangeKey *rkey;
  int            rkeyCnt;
  int             *fldNo;

  int *fldFixPos; /*ƹܤιԤˤ륹Ȱ*/
  int fixLen;     /*ƹܤθĹǼѿ*/
  int writePos;   /*ƹԤǡͤ񤭹*/
  char *repVal;   /*ִʸ*/
  int valLen;     /*ͤĹ*/
  char buf[MssFieldMaxLen]; /*񤭽ФѥХåե*/
  int i,j,k;

/*----------------------------------------------------------------------------*/
/*                                                                      */
/*----------------------------------------------------------------------------*/
  mssInit(argc,argv,&comHelp);       /* ʥʤɤν              */
  mssHelpDoc(opt,&comHelp,argc,argv);/* إ                                */
  mssSetOption(opt,argc,argv);       /* ޥɥץ              */
  fpr=mssOpenFPU(optINF.str,16);     /* ϥե륪ץ                  */

  /*եå*/
  /*
  flstat=flock( fileno(fpr->fp), LOCK_EX);
  if(flstat!=0){
    mssShowErrMsg("cannot lock the file");
    exit(mssErrorNoDefault);
  }
  */

  hdi=mssReadHeader(fpr);            /* إåɤ߹                      */
  mssSetOptKey(&optKEY, hdi);        /* -k ܤإåܤ˴ϢŤ     */
  mssSetOptFld(&optFLD, hdi);        /* -f ܤإåܤ˴ϢŤ     */

/*mssShowOption(opt);*/
    
  /*ϰϤǸǰʳϥ顼*/
  for(i=0; i<optVAL.cnt; i++){
    for(j=0; j<*(optVAL.valCnt+i)-1; j++){
      if( *(*(optVAL.to+i)+j) != NULL ){
        mssShowErrMsg("range can specify only at the last element on -v");
        exit(mssErrorNoDefault);
      }
    }
  }

  /*ͤΥå*/
  fldNo=mssMalloc(sizeof(int)*optKEY.cnt,"xtupdate");
  for(i=0; i<optKEY.cnt; i++){
    *(fldNo+i)=MssFlds2num(optKEY.flds,i);
  }
  rkeyCnt=optVAL.cnt;
  rkey=mssMalloc(sizeof(struct RangeKey)*rkeyCnt,"xtupdate");
  for(i=0; i<optVAL.cnt; i++){
    (rkey+i)->fldNo =fldNo;
    (rkey+i)->cnt   =*(optVAL.valCnt+i);
    (rkey+i)->str   =*(optVAL.from+i);
    (rkey+i)->from  =*(*(optVAL.from+i)+(rkey+i)->cnt-1);
    (rkey+i)->to    =*(*(optVAL.to  +i)+(rkey+i)->cnt-1);
    if((rkey+i)->to == NULL){
      (rkey+i)->to  = (rkey+i)->from;
    }
  }

  /*Υå*/
  for(i=0; i<optKEY.flds->cnt; i++){
    if(i+1 != MssFlds2priority(hdi->flds,MssFlds2num(optKEY.flds,i))  ){    
      mssShowErrMsg("key field(s) on -k mismatch with key field(s) on input data");
      exit(mssErrorNoDefault);
    }else if( 1==MssFlds2revFlg(hdi->flds,MssFlds2num(optKEY.flds,i)) ||
              1==MssFlds2numFlg(hdi->flds,MssFlds2num(optKEY.flds,i)) ){
      mssShowErrMsg("key field(s) on input data must not be sorted by numeric or reverse order");
      exit(mssErrorNoDefault);
    }
  }

  /*ִͤĹĹĶƤʤå*/
  for(i=0; i<optFLD.flds->cnt; i++){
    if( MssFlds2length(hdi->flds,MssFlds2num(optFLD.flds,i)) <
        strlen(*(optFLD.fldOpt+i)) ) {
      mssShowErrMsg("length of update value exceed the fixd length of field");
      exit(mssErrorNoDefault);
    }
  }
  
  /*쥳ɤĹ*/
  recLen=0; 
  for(i=0; i<hdi->flds->cnt; i++){
    if( MssFlds2length(hdi->flds,i)==0) {
      mssShowErrMsg("input data is not fixed lenght");                      
      exit(mssErrorNoDefault);
    }
    recLen += MssFlds2length(hdi->flds,i);
    recLen++; /*ǥߥȲʬ*/                                            
  }

  /*ǡƬ֤򥻥å*/
  datTop=fpr->chrCnt;

  /*ǡԿΥå*/
  recCnt=hdi->recCnt;
  if(recCnt==-1) {
    mssShowErrMsg("there is no information of records count");
    exit(mssErrorNoDefault);
  }

  /*ƹܤΰ֤׻*/
  fldFixPos=mssMalloc(sizeof(int)*hdi->flds->cnt,"fldpos");
  *(fldFixPos+0)=0;
  for(i=1; i<hdi->flds->cnt; i++){
    *(fldFixPos+i)=*(fldFixPos+i-1)+MssFlds2length(hdi->flds,i-1)+1;
  }

/*----------------------------------------------------------------------------*/
/*ᥤ롼                                                              */
/*----------------------------------------------------------------------------*/
  fr =mssInitFldRec(hdi->flds->cnt);
  /* -v ǿ֤*/
  for(i=0; i<rkeyCnt; i++){
    recNum=bSearch(rkey+i,fpr,hdi->flds->cnt,recCnt,datTop,recLen);
    while(1){
      if(recNum>=recCnt) break;
      mssReadRandFldRec(fpr,fr,recNum,datTop,recLen);
      mssGV.inCnt++;
      if(!inRange(fr->pnt,rkey+i)) break;

      /*Ͻ*/
      for(j=0; j<optFLD.flds->cnt; j++){

        /*ܤĹ(Ĺ)*/
        fixLen=MssFlds2length(hdi->flds,MssFlds2num(optFLD.flds,j));

        /*񤭽Ф(Ƭ)*/
        writePos=datTop+recLen*recNum+*(fldFixPos+MssFlds2num(optFLD.flds,j));

        /*ִʸ*/
        repVal=*(optFLD.fldOpt+j);

        /*ִʸĹ*/
        valLen=strlen(repVal);

        /*ޤΰ򥹥ڡ\0ǽ*/
        for(k=0; k<fixLen; k++){
          buf[k]=' ';
        }
        buf[k]='\0';

        /*ʸ򥳥ԡ*/
        strncpy(&buf[fixLen-valLen],repVal,valLen);

        /*ǥݥ󥿤ưХåեƤ񤭹*/
        fseek(fpr->fp,writePos,SEEK_SET);
        fputs(buf,fpr->fp);
      }
      mssGV.outCnt++;
      recNum++;
    }
  }
  mssFree(fpr->curPnt);
  mssFreeFldRec(fr);
  mssFree(fldNo);
  mssFree(rkey);

  /*ե륢å*/
  /*
  flock( fileno(fpr->fp), LOCK_UN);
  */

/*----------------------------------------------------------------------------*/
/*եå&λ                                                       */
/*----------------------------------------------------------------------------*/
  mssCloseFPU(fpr);       /*ϥեΥ      */
  mssFreeHeader(hdi);     /*ϥإåΰ賫          */
  mssFreeOption(opt);     /*ץΰ賫          */
  mssShowEndMsg();        /* λå             */
  mssEnd(mssExitSuccess); /* λ                       */
  return(0);              /* to avoid warning message   */
}

==> musashi-core-1.0.4.tar.gz: musashi-core-1.0.4/exp/xtbsupdate.c <==
/*============================================================================*/
/* ѹ                                                                   */
/*----------------------------------------------------------------------------*/
/* 0.1 : ǿ 2003/06/20                                              */
/* 0.2 : ץͿѹ 2003/08/18                                  */
/* 0.3 : եå(¾å)ǽɲ 2003/09/26                        */
/*============================================================================*/

#include <musashi.h>
#include <stdlib.h>
#include <string.h>
#include <sys/file.h>

#include <xtbsupdateHelp.h>
struct mssComHelp comHelp={
  "xtbsupdate",   /* ޥ̾       */
  "0.3",          /* С       */
  HELPT,          /* ޥɥȥ */
  HELPS,          /*              */
  HELPE,          /*            */
  HELPR,          /* ȥޥ     */
  HELPA,          /* Ծ         */
  HELPB,          /* ХݡȾ */
  HELPH           /* ۡڡ     */
};

extern struct mssGlobalVariables mssGV;

/**
 * ϰϥι¤
 * ϰϤǤΤϡǸʸΤ
 * ex) 
 */
struct RangeKey {
  int   *fldNo; /*ƹܤֹ*/
  char  **str;
  char  *from;
  char  *to;
  int    cnt;
};

int keyCmp(char **str1, char **str2, struct RangeKey *rkey){
  int i;
  int cmp=0;

  for(i=0; i<rkey->cnt; i++){
    cmp=strcmp( *(str1+*(rkey->fldNo+i)), *(str2+*(rkey->fldNo+i)) );
               /* ǡ         -kιܥץ */
    if(cmp!=0) return(cmp);
  }
  return(cmp);
}

int rngKeyCmp(char **str, struct RangeKey *rkey){
  int i;
  int cmp=0;

  for(i=0; i<rkey->cnt; i++){
    if( MssIsNull(*(str+*(rkey->fldNo+i))) ) return(1);
    cmp=strcmp( *(rkey->str+i), *(str+*(rkey->fldNo+i)) );
               /* ǡ         -kιܥץ */
    if(cmp!=0) return(cmp);
  }
  return(cmp);
}

/**
 * Ϳ줿쥳(pnt)ϰϥ(rkey)ˤ뤫ɤå
 * ϰʤ1,ǤʤУ֤
 * ǡNULLʤ飰֤
 */
int inRange(char **pnt,struct RangeKey *rkey){
  int i;
  int cmp;
  int cmpFrom;
  int cmpTo;

  /*ϰϹܰʳunmatchʤ0֤*/
  for(i=0; i<rkey->cnt-1; i++){
    if( MssIsNull(*(pnt+*(rkey->fldNo+i))) ) return(0);
    cmp=strcmp( *(rkey->str+i), *(pnt+*(rkey->fldNo+i)) );
    if(cmp!=0) return(0);
  }

  /*ϰϹʤ1֤*/
  if( MssIsNull(*(pnt+*(rkey->fldNo+i))) ) return(0);
  cmpFrom=strcmp( rkey->from, *(pnt+*(rkey->fldNo+i)) );
  cmpTo  =strcmp( rkey->to,   *(pnt+*(rkey->fldNo+i)) );
  if( cmpFrom<=0 && cmpTo>=0){
    return(1);
  }else{
    return(0);
  }
}

/**
 * ǡ顢ϰϥƬ֤֤
 */
int bSearch(struct RangeKey *rkey,struct mssFPR *fp,int fldCnt,int recCnt,int datTop,int recLen){

  struct mssFldRec    *fr1;  /*-ԥХåե¤             */
  struct mssFldRec    *fr2;  /*-ԥХåե¤             */
  int l=0;
  int r=recCnt-1;
  int x;
  int cmp;
  char *tmp;

  fr1 =mssInitFldRec(fldCnt);
  fr2 =mssInitFldRec(fldCnt);

  while(r>=l){
    x=(l+r)/2;
    mssReadRandFldRec(fp,fr1,x,datTop,recLen);
    tmp=fp->curPnt; /* ɲ 2004/10/31 */
    mssGV.inCnt++;
    cmp=rngKeyCmp(fr1->pnt,rkey);

         /* v<x */
         if(cmp<0) r=x-1;
         /* v>x */
    else if(cmp>0) l=x+1;
         /* v=x */
    else          {
      if(x==0){
        mssFree(fp->curPnt); /* ɲ 2004/10/31 */
	mssFreeFldRec(fr1);  /* ɲ 2004/10/31 */
	mssFreeFldRec(fr2);  /* ɲ 2004/10/31 */
        return(x);
      }
      mssReadRandFldRec(fp,fr2,x-1,datTop,recLen);
      if(0!=keyCmp(fr1->pnt, fr2->pnt, rkey)){
        mssFree(fp->curPnt); /* ɲ 2004/10/31 */
        mssFree(tmp);        /* ɲ 2004/10/31 */
        mssFreeFldRec(fr1);
        mssFreeFldRec(fr2);
        return(x);
      }else{
        r=x-1;
        mssFree(fp->curPnt); /* ɲ 2004/10/31 */
      }
    }
    mssFree(tmp); /* ɲ 2004/10/31 */
  }
  mssFreeFldRec(fr1);
  mssFreeFldRec(fr2);
  return(l);
}

int main(int argc, char *argv[]){
/*============================================================================*/
/* ץ                                                       */
/*============================================================================*/
/*----------------------------------------------------------------------------*/
/*                                                                    */
/*----------------------------------------------------------------------------*/
  MssOptKEY optKEY={
    OKEY,   /* ץ󥿥                                             */
    "k",    /* (ʣʸԲ)                                   */
    0,      /* 0:ץ, 1:ɬ, 2:XMLtableǤΤɬ(txtǤ̵)      */
    MssFieldMaxCnt, /* ǽʺܿ                                 */
    "i",    /* оݤȤϥǡΥ(GUI)                  */
    1,      /* ǥե(Υץ󤬻ꤵʤäȤư) */
            /* 1:ƤιԤۤ륭ͤȤư                             */
            /* 2:ƤιԤƱͤȤư)                            */
    KEYT,   /* ΥץΥȥ(Helpɽ)                         */
    KEYC    /* ΥץΥ(Helpɽ)                         */
  };

/*----------------------------------------------------------------------------*/
/* ʸꥹ                                                           */
/*----------------------------------------------------------------------------*/
  MssOptSRL optVAL={
    OSRL,   /* ץ󥿥                                             */
    "v",    /* (ʣʸԲ)                                   */
    1,      /* 0:ץ, 1:ɬ, 2:XMLtableǤΤɬ(txtǤ̵)      */
    NULL,   /* ǥե                                                   */
    100,    /*()κ                                      */
    MssFieldMaxCnt,/*Ǥ(޶)κ                       */
    1,      /*ͤκǾʸ                                                */
    MssFieldMaxLen,/*ͤκʸ                                         */
    1,      /*ϰϻǽե饰                                            */
    VALT,   /* ΥץΥȥ(Helpɽ)                         */
    VALC    /* ΥץΥ(Helpɽ)                         */
  };

/*----------------------------------------------------------------------------*/
/*                                                                    */
/*----------------------------------------------------------------------------*/
  MssOptFLD optFLD={
    OFLD,   /* ץ󥿥                                             */
    "f",    /* (ʣʸԲ)                                   */
    1,      /* 0:ץ, 1:ɬ, 2:XMLtableǤΤɬ(txtǤ̵)      */
    MssFieldMaxCnt, /* ǽʺܿ                                 */
    "i",    /* оݤȤϥǡΥ(GUI)                  */
    0,      /* ɽĤ뤫ɤ(0:Բ,1:)                      */
    0,      /* ̾Ǥ뤫ɤ(0:Բ,1:)                    */
    NULL,   /* ܥץ(%ʲ)ǻǽʸ                        */
            /* ex) ԲĤξNULL, "nr": "-f ̾%rn"λǽ     */
    FLDT,   /* ΥץΥȥ(Helpɽ)                         */
    FLDC,   /* ΥץΥ(Helpɽ)                         */
    NULL    /* ե饰ˤĤƤ(Helpɽ)ʣξϥޤǶڤ   */
  };

/*----------------------------------------------------------------------------*/
/* ϥե                                                               */
/*----------------------------------------------------------------------------*/
  MssOptINF optINF={
    OINF,   /* ץ󥿥                                             */
    "i",    /* (ʣʸԲ)                                   */
    1,      /* 0:ץ, 1:ɬ                                         */
    1,      /* ǽκե                                     */
    0,      /*1:file not foundΥ顼ǽλʤ 0:                   */
    INFT,   /* ΥץΥȥ(Helpɽ)                         */
    INFC    /* ΥץΥ(Helpɽ)                         */
  };

/*----------------------------------------------------------------------------*/
/* ץޤȤ                                                       */
/*----------------------------------------------------------------------------*/
  void *opt[]={&optKEY,&optVAL,&optFLD,&optINF,NULL};

/*============================================================================*/
/* ѿ                                                             */
/*============================================================================*/
  struct mssHeader    *hdi; /*ϥե<head>Ǽ¤*/
  struct mssFPR       *fpr; /*ϥե빽¤                */
  struct mssFldRec    *fr;  /*-ԥХåե¤             */

  int recLen;     /*쥳Ĺ*/
  int recCnt;     /*쥳ɿ*/
  int datTop;     /*ǡΤƬΰ*/
  int recNum;     /*̤ȤƤΥ쥳ֹ*/
  /*int flstat;*/     /*եåΥơ*/

  struct RangeKey *rkey;
  int            rkeyCnt;
  int             *fldNo;

  int *fldFixPos; /*ƹܤιԤˤ륹Ȱ*/
  int fixLen;     /*ƹܤθĹǼѿ*/
  int writePos;   /*ƹԤǡͤ񤭹*/
  char *repVal;   /*ִʸ*/
  int valLen;     /*ͤĹ*/
  char buf[MssFieldMaxLen]; /*񤭽ФѥХåե*/
  int i,j,k;
  char *tmp;

/*----------------------------------------------------------------------------*/
/*                                                                      */
/*----------------------------------------------------------------------------*/
  mssInit(argc,argv,&comHelp);       /* ʥʤɤν              */
  mssHelpDoc(opt,&comHelp,argc,argv);/* إ                                */
  mssSetOption(opt,argc,argv);       /* ޥɥץ              */
  fpr=mssOpenFPU(optINF.str,16);     /* ϥե륪ץ                  */

  /*եå*/
  /*
  flstat=flock( fileno(fpr->fp), LOCK_EX);
  if(flstat!=0){
    mssShowErrMsg("cannot lock the file");
    exit(mssErrorNoDefault);
  }
  */

  hdi=mssReadHeader(fpr);            /* إåɤ߹                      */
  mssSetOptKey(&optKEY, hdi);        /* -k ܤإåܤ˴ϢŤ     */
  mssSetOptFld(&optFLD, hdi);        /* -f ܤإåܤ˴ϢŤ     */

/*mssShowOption(opt);*/
    
  /*ϰϤǸǰʳϥ顼*/
  for(i=0; i<optVAL.cnt; i++){
    for(j=0; j<*(optVAL.valCnt+i)-1; j++){
      if( *(*(optVAL.to+i)+j) != NULL ){
        mssShowErrMsg("range can specify only at the last element on -v");
        exit(mssErrorNoDefault);
      }
    }
  }

  /*ͤΥå*/
  fldNo=mssMalloc(sizeof(int)*optKEY.cnt,"xtupdate");
  for(i=0; i<optKEY.cnt; i++){
    *(fldNo+i)=MssFlds2num(optKEY.flds,i);
  }
  rkeyCnt=optVAL.cnt;
  rkey=mssMalloc(sizeof(struct RangeKey)*rkeyCnt,"xtupdate");
  for(i=0; i<optVAL.cnt; i++){
    (rkey+i)->fldNo =fldNo;
    (rkey+i)->cnt   =*(optVAL.valCnt+i);
    (rkey+i)->str   =*(optVAL.from+i);
    (rkey+i)->from  =*(*(optVAL.from+i)+(rkey+i)->cnt-1);
    (rkey+i)->to    =*(*(optVAL.to  +i)+(rkey+i)->cnt-1);
    if((rkey+i)->to == NULL){
      (rkey+i)->to  = (rkey+i)->from;
    }
  }

  /*Υå*/
  for(i=0; i<optKEY.flds->cnt; i++){
    if(i+1 != MssFlds2priority(hdi->flds,MssFlds2num(optKEY.flds,i))  ){    
      mssShowErrMsg("key field(s) on -k mismatch with key field(s) on input data");
      exit(mssErrorNoDefault);
    }else if( 1==MssFlds2revFlg(hdi->flds,MssFlds2num(optKEY.flds,i)) ||
              1==MssFlds2numFlg(hdi->flds,MssFlds2num(optKEY.flds,i)) ){
      mssShowErrMsg("key field(s) on input data must not be sorted by numeric or reverse order");
      exit(mssErrorNoDefault);
    }
  }

  /*ִͤĹĹĶƤʤå*/
  for(i=0; i<optFLD.flds->cnt; i++){
    if( MssFlds2length(hdi->flds,MssFlds2num(optFLD.flds,i)) <
        strlen(*(optFLD.fldOpt+i)) ) {
      mssShowErrMsg("length of update value exceed the fixd length of field");
      exit(mssErrorNoDefault);
    }
  }
  
  /*쥳ɤĹ*/
  recLen=0; 
  for(i=0; i<hdi->flds->cnt; i++){
    if( MssFlds2length(hdi->flds,i)==0) {
      mssShowErrMsg("input data is not fixed lenght");                      
      exit(mssErrorNoDefault);
    }
    recLen += MssFlds2length(hdi->flds,i);
    recLen++; /*ǥߥȲʬ*/                                            
  }

  /*ǡƬ֤򥻥å*/
  datTop=fpr->chrCnt;

  /*ǡԿΥå*/
  recCnt=hdi->recCnt;
  if(recCnt==-1) {
    mssShowErrMsg("there is no information of records count");
    exit(mssErrorNoDefault);
  }

  /*ƹܤΰ֤׻*/
  fldFixPos=mssMalloc(sizeof(int)*hdi->flds->cnt,"fldpos");
  *(fldFixPos+0)=0;
  for(i=1; i<hdi->flds->cnt; i++){
    *(fldFixPos+i)=*(fldFixPos+i-1)+MssFlds2length(hdi->flds,i-1)+1;
  }

/*----------------------------------------------------------------------------*/
/*ᥤ롼                                                              */
/*----------------------------------------------------------------------------*/
  fr =mssInitFldRec(hdi->flds->cnt);
  /* -v ǿ֤*/
  for(i=0; i<rkeyCnt; i++){
    recNum=bSearch(rkey+i,fpr,hdi->flds->cnt,recCnt,datTop,recLen);
    while(1){
      if(recNum>=recCnt) break;
      mssReadRandFldRec(fpr,fr,recNum,datTop,recLen);
      tmp=fpr->curPnt;
      mssGV.inCnt++;
      if(!inRange(fr->pnt,rkey+i)){
        mssFree(tmp); /* ɲ 2004/10/31 */
        break;
      }

      /*Ͻ*/
      for(j=0; j<optFLD.flds->cnt; j++){

        /*ܤĹ(Ĺ)*/
        fixLen=MssFlds2length(hdi->flds,MssFlds2num(optFLD.flds,j));

        /*񤭽Ф(Ƭ)*/
        writePos=datTop+recLen*recNum+*(fldFixPos+MssFlds2num(optFLD.flds,j));

        /*ִʸ*/
        repVal=*(optFLD.fldOpt+j);

        /*ִʸĹ*/
        valLen=strlen(repVal);

        /*ޤΰ򥹥ڡ\0ǽ*/
        for(k=0; k<fixLen; k++){
          buf[k]=' ';
        }
        buf[k]='\0';

        /*ʸ򥳥ԡ*/
        strncpy(&buf[fixLen-valLen],repVal,valLen);

        /*ǥݥ󥿤ưХåեƤ񤭹*/
        fseek(fpr->fp,writePos,SEEK_SET);
        fputs(buf,fpr->fp);
      }
      mssGV.outCnt++;
      recNum++;
      mssFree(tmp); /* ɲ 2004/10/31 */
    }
  }
  /* mssFree(fpr->curPnt); */
  mssFreeFldRec(fr);
  mssFree(fldNo);
  mssFree(rkey);
  mssFree(fldFixPos);

  /*ե륢å*/
  /*
  flock( fileno(fpr->fp), LOCK_UN);
  */

/*----------------------------------------------------------------------------*/
/*եå&λ                                                       */
/*----------------------------------------------------------------------------*/
  mssCloseFPU(fpr);       /*ϥեΥ      */
  mssFreeHeader(hdi);     /*ϥإåΰ賫          */
  mssFreeOption(opt);     /*ץΰ賫          */
  mssShowEndMsg();        /* λå             */
  mssEnd(mssExitSuccess); /* λ                       */
  return(0);              /* to avoid warning message   */
}
